@@ -24,6 +24,39 @@ int uwsgi_simple_wait_read_hook(int fd, int timeout) {
         return ret;
 }
 
+int uwsgi_simple_wait_read2_hook(int fd0, int fd1, int timeout, int *fd) {
+        struct pollfd upoll[2];
+        timeout = timeout * 1000;
+
+        upoll[0].fd = fd0;
+        upoll[0].events = POLLIN;
+        upoll[0].revents = 0;
+
+        upoll[1].fd = fd1;
+        upoll[1].events = POLLIN;
+        upoll[1].revents = 0;
+
+        int ret = poll(upoll, 2, timeout);
+
+        if (ret > 0) {
+                if (upoll[0].revents & POLLIN) {
+			*fd = fd0;
+                        return 1;
+                }
+                if (upoll[1].revents & POLLIN) {
+			*fd = fd1;
+                        return 1;
+                }
+                return -1;
+        }
+        if (ret < 0) {
+                uwsgi_error("uwsgi_simple_wait_read_hook2()/poll()");
+        }
+
+        return ret;
+}
+
+
 /*
 	seek()/rewind() language-independent implementations.
 */
@@ -32,14 +65,16 @@ void uwsgi_request_body_seek(struct wsgi_request *wsgi_req, off_t pos) {
 	if (wsgi_req->post_file) {
 		if (pos < 0) {
 			if (fseek(wsgi_req->post_file, pos, SEEK_CUR)) {
-                        	uwsgi_error("uwsgi_request_body_seek()/fseek()");
+                        	uwsgi_req_error("uwsgi_request_body_seek()/fseek()");
+				wsgi_req->read_errors++;
                 	}
 			wsgi_req->post_pos = ftell(wsgi_req->post_file);
 			return;
 		}
 
 		if (fseek(wsgi_req->post_file, pos, SEEK_SET)) {
-			uwsgi_error("uwsgi_request_body_seek()/fseek()");
+			uwsgi_req_error("uwsgi_request_body_seek()/fseek()");
+			wsgi_req->read_errors++;
 		}
 		wsgi_req->post_pos = ftell(wsgi_req->post_file);
 		return;
@@ -95,7 +130,7 @@ static int consume_body_for_readline(struct wsgi_request *wsgi_req) {
 		if (wsgi_req->post_readline_size - wsgi_req->post_readline_watermark < remains) {
         		char *tmp_buf = realloc(wsgi_req->post_readline_buf, wsgi_req->post_readline_size + remains);
                 	if (!tmp_buf) {
-                		uwsgi_error("consume_body_for_readline()/realloc()");
+                		uwsgi_req_error("consume_body_for_readline()/realloc()");
                         	return -1;
                 	}
                 	wsgi_req->post_readline_buf = tmp_buf;
@@ -114,7 +149,7 @@ static int consume_body_for_readline(struct wsgi_request *wsgi_req) {
 	if (wsgi_req->post_file) {
 		size_t ret = fread(wsgi_req->post_readline_buf + wsgi_req->post_readline_watermark, remains, 1, wsgi_req->post_file);	
 		if (ret == 0) {
-			uwsgi_error("consume_body_for_readline()/fread()");
+			uwsgi_req_error("consume_body_for_readline()/fread()");
 			return -1;
 		}
 		wsgi_req->post_pos += remains;
@@ -140,6 +175,7 @@ static int consume_body_for_readline(struct wsgi_request *wsgi_req) {
 	}
 	if (len == 0) {
 		uwsgi_read_error(remains);
+		wsgi_req->read_errors++;
 		return -1;	
 	}
 	if (len < 0) {
@@ -147,6 +183,7 @@ static int consume_body_for_readline(struct wsgi_request *wsgi_req) {
 			goto wait;
 		}
 		uwsgi_read_error(remains);
+		wsgi_req->read_errors++;
 		return -1;
 	}
 wait:
@@ -159,6 +196,7 @@ wait:
 			return 0;
 		}
 		uwsgi_read_error(remains);
+		wsgi_req->read_errors++;
                 return -1;
 	}
         // 0 means timeout
@@ -167,6 +205,7 @@ wait:
 		return -1;
         }
 	uwsgi_read_error(remains);
+	wsgi_req->read_errors++;
         return -1;
 }
 
@@ -204,7 +243,8 @@ char *uwsgi_request_body_readline(struct wsgi_request *wsgi_req, ssize_t hint, s
 		size_t amount = UMIN(uwsgi.buffer_size, wsgi_req->post_cl);
 		wsgi_req->post_readline_buf = malloc(amount);
 		if (!wsgi_req->post_readline_buf) {
-			uwsgi_error("uwsgi_request_body_readline()/malloc()");
+			uwsgi_req_error("uwsgi_request_body_readline()/malloc()");
+			wsgi_req->read_errors++;
 			*rlen = -1;
 			return NULL;
 		}
@@ -217,6 +257,7 @@ char *uwsgi_request_body_readline(struct wsgi_request *wsgi_req, ssize_t hint, s
 		if (wsgi_req->post_pos >= wsgi_req->post_cl) break;
 
                         if (consume_body_for_readline(wsgi_req)) {
+				wsgi_req->read_errors++;
                                 *rlen = -1;
                                 return NULL;
                         }
@@ -272,8 +313,9 @@ char *uwsgi_request_body_read(struct wsgi_request *wsgi_req, ssize_t hint, ssize
 			if (avail > wsgi_req->post_read_buf_size) {
 				char *tmp_buf = realloc(wsgi_req->post_read_buf, avail);
 				if (!tmp_buf) {
-                                	uwsgi_error("uwsgi_request_body_read()/realloc()");
+                                	uwsgi_req_error("uwsgi_request_body_read()/realloc()");
                                 	*rlen = -1;
+					wsgi_req->read_errors++;
                                 	return NULL;
                         	}
                         	wsgi_req->post_read_buf = tmp_buf;
@@ -320,7 +362,8 @@ char *uwsgi_request_body_read(struct wsgi_request *wsgi_req, ssize_t hint, ssize
 	if (!wsgi_req->post_read_buf) {
 		wsgi_req->post_read_buf = malloc(remains);
 		if (!wsgi_req->post_read_buf) {
-			uwsgi_error("uwsgi_request_body_read()/malloc()");
+			uwsgi_req_error("uwsgi_request_body_read()/malloc()");
+			wsgi_req->read_errors++;
 			*rlen = -1;
 			return NULL;
 		}
@@ -331,7 +374,8 @@ char *uwsgi_request_body_read(struct wsgi_request *wsgi_req, ssize_t hint, ssize
 		if ((remains+*rlen) > wsgi_req->post_read_buf_size) {
 			char *tmp_buf = realloc(wsgi_req->post_read_buf, (remains+*rlen));
 			if (!tmp_buf) {
-				uwsgi_error("uwsgi_request_body_read()/realloc()");
+				uwsgi_req_error("uwsgi_request_body_read()/realloc()");
+				wsgi_req->read_errors++;
 				*rlen = -1;
 				return NULL;
 			}
@@ -348,7 +392,8 @@ char *uwsgi_request_body_read(struct wsgi_request *wsgi_req, ssize_t hint, ssize
 	if (wsgi_req->post_file) {
 		if (fread(wsgi_req->post_read_buf + *rlen, remains, 1, wsgi_req->post_file) != 1) {
 			*rlen = -1;
-			uwsgi_error("uwsgi_request_body_read()/fread()");
+			uwsgi_req_error("uwsgi_request_body_read()/fread()");
+			wsgi_req->read_errors++;
 			return NULL;
 		}
 		*rlen += remains;
@@ -378,6 +423,7 @@ char *uwsgi_request_body_read(struct wsgi_request *wsgi_req, ssize_t hint, ssize
 			}
 			*rlen = -1;
 			uwsgi_read_error(remains);
+			wsgi_req->read_errors++;
 			return NULL;
 		}
 wait:
@@ -398,6 +444,7 @@ wait:
 			}
 			else {
 				uwsgi_read_error(remains);
+				wsgi_req->read_errors++;
 			}
 			return NULL;
 		}
@@ -409,6 +456,7 @@ wait:
 		}
 		*rlen = -1;
 		uwsgi_read_error(remains);
+		wsgi_req->read_errors++;
 		return NULL;
 	}
 
@@ -447,6 +495,7 @@ int uwsgi_postbuffer_do_in_mem(struct wsgi_request *wsgi_req) {
                                 goto wait;
                         }
 			uwsgi_read_error(remains);
+			wsgi_req->read_errors++;
                         return -1;
                 }
 
@@ -462,6 +511,7 @@ wait:
 		}
                 if (ret < 0) {
 			uwsgi_read_error(remains);
+			wsgi_req->read_errors++;
                         return -1;
                 }
 		uwsgi_read_timeout(remains);
@@ -482,7 +532,8 @@ int uwsgi_postbuffer_do_in_disk(struct wsgi_request *wsgi_req) {
 
         wsgi_req->post_file = uwsgi_tmpfile();
         if (!wsgi_req->post_file) {
-                uwsgi_error("uwsgi_postbuffer_do_in_disk()/uwsgi_tmpfile()");
+                uwsgi_req_error("uwsgi_postbuffer_do_in_disk()/uwsgi_tmpfile()");
+		wsgi_req->read_errors++;
                 return -1;
         }
 
@@ -518,6 +569,7 @@ int uwsgi_postbuffer_do_in_disk(struct wsgi_request *wsgi_req) {
                                 goto wait;
                         }
 			uwsgi_read_error(remains);
+			wsgi_req->read_errors++;
                         goto end;
                 }
 
@@ -531,11 +583,13 @@ wait:
 			}
 			else {
 				uwsgi_read_error(remains);
+				wsgi_req->read_errors++;
 			}
                         goto end;
 		}
                 if (ret < 0) {
 			uwsgi_read_error(remains);
+			wsgi_req->read_errors++;
                         goto end;
                 }
 		uwsgi_read_timeout(remains);
@@ -543,7 +597,8 @@ wait:
 
 write:
                 if (fwrite(wsgi_req->post_buffering_buf, rlen, 1, wsgi_req->post_file) != 1) {
-                        uwsgi_error("uwsgi_postbuffer_do_in_disk()/fwrite()");
+                        uwsgi_req_error("uwsgi_postbuffer_do_in_disk()/fwrite()");
+			wsgi_req->read_errors++;
                         goto end;
                 }
 
