@@ -1066,6 +1066,27 @@ PyObject *py_uwsgi_websocket_send(PyObject * self, PyObject * args) {
         return Py_None;
 }
 
+PyObject *py_uwsgi_websocket_send_binary(PyObject * self, PyObject * args) {
+        char *message = NULL;
+        Py_ssize_t message_len = 0;
+
+        if (!PyArg_ParseTuple(args, "s#:websocket_send_binary", &message, &message_len)) {
+                return NULL;
+        }
+
+        struct wsgi_request *wsgi_req = py_current_wsgi_req();
+
+        UWSGI_RELEASE_GIL
+        int ret  = uwsgi_websocket_send_binary(wsgi_req, message, message_len);
+        UWSGI_GET_GIL
+        if (ret < 0) {
+                return PyErr_Format(PyExc_IOError, "unable to send websocket binary message");
+        }
+        Py_INCREF(Py_None);
+        return Py_None;
+}
+
+
 PyObject *py_uwsgi_chunked_read(PyObject * self, PyObject * args) {
 	int timeout = 0; 
 	if (!PyArg_ParseTuple(args, "|i:chunked_read", &timeout)) {
@@ -1426,222 +1447,228 @@ PyObject *py_uwsgi_extract(PyObject * self, PyObject * args) {
 }
 
 
-PyObject *py_uwsgi_sharedarea_inclong(PyObject * self, PyObject * args) {
+PyObject *py_uwsgi_sharedarea_inc64(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
-	uint64_t value = 1;
-	uint64_t current_value = 0;
+	int64_t value = 1;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "l|l:sharedarea_inclong", &pos, &value)) {
+	if (!PyArg_ParseTuple(args, "il|l:sharedarea_inc64", &id, &pos, &value)) {
 		return NULL;
 	}
 
-	if (pos + 8 >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-	
 	UWSGI_RELEASE_GIL
-
-	uwsgi_wlock(uwsgi.sa_lock);
-
-	memcpy(&current_value, uwsgi.sharedarea + pos, 8);
-	value = current_value + value;
-	memcpy(uwsgi.sharedarea + pos, &value, 8);
-
-        uwsgi_rwunlock(uwsgi.sa_lock);
+	int ret = uwsgi_sharedarea_inc64(id, pos, value);
 	UWSGI_GET_GIL
+	
+	if (ret) {
+		return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_inc64()");
+	}
 
-	return PyInt_FromLong(value);
+	Py_INCREF(Py_None);
+        return Py_None;
 
 }
 
-PyObject *py_uwsgi_sharedarea_writelong(PyObject * self, PyObject * args) {
+PyObject *py_uwsgi_sharedarea_write64(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
-	uint64_t value = 0;
+	int64_t value = 0;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "ll:sharedarea_writelong", &pos, &value)) {
+	if (!PyArg_ParseTuple(args, "ill:sharedarea_write64", &id, &pos, &value)) {
 		return NULL;
 	}
 
-	if (pos + 8 >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
 	UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_write64(id, pos, &value);
+        UWSGI_GET_GIL
 
-	uwsgi_wlock(uwsgi.sa_lock);
-
-	memcpy(uwsgi.sharedarea + pos, &value, 8);
-
-        uwsgi_rwunlock(uwsgi.sa_lock);
-
-	UWSGI_GET_GIL
-
-	return PyInt_FromLong(value);
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_write64()");
+        }
 
+	Py_INCREF(Py_None);
+        return Py_None;
 }
 
 PyObject *py_uwsgi_sharedarea_write(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
 	char *value;
 	Py_ssize_t value_len = 0;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "ls#:sharedarea_write", &pos, &value, &value_len)) {
+	if (!PyArg_ParseTuple(args, "ils#:sharedarea_write", &id, &pos, &value, &value_len)) {
 		return NULL;
 	}
 
-	if (pos + value_len >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
 	UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_write(id, pos, value, value_len);
+        UWSGI_GET_GIL
 
-	uwsgi_wlock(uwsgi.sa_lock);
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_write()");
+        }
 
-	memcpy(uwsgi.sharedarea + pos, value, value_len);
+        Py_INCREF(Py_None);
+        return Py_None;
+}
 
+PyObject *py_uwsgi_sharedarea_update(PyObject * self, PyObject * args) {
+        int id;
 
-	uwsgi_rwunlock(uwsgi.sa_lock);
+        if (!PyArg_ParseTuple(args, "i:sharedarea_update", &id)) {
+                return NULL;
+        }
 
-	UWSGI_GET_GIL
+        if (uwsgi_sharedarea_update(id)) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_update()");
+        }
 
-	return PyInt_FromLong(value_len);
-	
+        Py_INCREF(Py_None);
+        return Py_None;
 
 }
 
-PyObject *py_uwsgi_sharedarea_writebyte(PyObject * self, PyObject * args) {
-	uint64_t pos = 0;
-	char value;
+PyObject *py_uwsgi_sharedarea_rlock(PyObject * self, PyObject * args) {
+        int id;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+        if (!PyArg_ParseTuple(args, "i:sharedarea_rlock", &id)) {
+                return NULL;
+        }
 
+        UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_rlock(id);
+        UWSGI_GET_GIL
 
-	if (!PyArg_ParseTuple(args, "lb:sharedarea_writebyte", &pos, &value)) {
-		return NULL;
-	}
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_rlock()");
+        }
 
-	if (pos >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	UWSGI_RELEASE_GIL
-
-	uwsgi_wlock(uwsgi.sa_lock);
-
-	uwsgi.sharedarea[pos] = value;
-
-
-	uwsgi_rwunlock(uwsgi.sa_lock);
-
-	UWSGI_GET_GIL
-
-	return PyInt_FromLong(value);
+        Py_INCREF(Py_None);
+        return Py_None;
 
 }
 
-PyObject *py_uwsgi_sharedarea_readlong(PyObject * self, PyObject * args) {
+PyObject *py_uwsgi_sharedarea_wlock(PyObject * self, PyObject * args) {
+        int id;
+
+        if (!PyArg_ParseTuple(args, "i:sharedarea_wlock", &id)) {
+                return NULL;
+        }
+
+        UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_wlock(id);
+        UWSGI_GET_GIL
+
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_wlock()");
+        }
+
+        Py_INCREF(Py_None);
+        return Py_None;
+
+}
+
+PyObject *py_uwsgi_sharedarea_unlock(PyObject * self, PyObject * args) {
+        int id;
+
+        if (!PyArg_ParseTuple(args, "i:sharedarea_unlock", &id)) {
+                return NULL;
+        }
+
+        UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_unlock(id);
+        UWSGI_GET_GIL
+
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_unlock()");
+        }
+
+        Py_INCREF(Py_None);
+        return Py_None;
+
+}
+
+
+
+
+PyObject *py_uwsgi_sharedarea_write8(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
-	uint64_t value;
+	int8_t value;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "l:sharedarea_readlong", &pos)) {
+	if (!PyArg_ParseTuple(args, "ilb:sharedarea_write8", &id, &pos, &value)) {
 		return NULL;
 	}
 
-	if (pos + 8 >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
+	UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_write8(id, pos, &value);
+        UWSGI_GET_GIL
+
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_write8()");
+        }
+
+        Py_INCREF(Py_None);
+        return Py_None;
+
+}
+
+PyObject *py_uwsgi_sharedarea_read64(PyObject * self, PyObject * args) {
+	int id;
+	uint64_t pos = 0;
+	int64_t value;
+
+	if (!PyArg_ParseTuple(args, "il:sharedarea_read64", &id, &pos)) {
+		return NULL;
 	}
 
 	UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_read64(id, pos, &value);
+        UWSGI_GET_GIL
 
-	uwsgi_wlock(uwsgi.sa_lock);
-
-	memcpy(&value, uwsgi.sharedarea + pos, 8);
-
-
-	uwsgi_rwunlock(uwsgi.sa_lock);
-
-	UWSGI_GET_GIL
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_read64()");
+        }
 
 	return PyLong_FromLong(value);
 
 }
 
-
-PyObject *py_uwsgi_sharedarea_readbyte(PyObject * self, PyObject * args) {
+PyObject *py_uwsgi_sharedarea_read8(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
+	int8_t byte;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "l:sharedarea_readbyte", &pos)) {
+	if (!PyArg_ParseTuple(args, "il:sharedarea_read8", &id, &pos)) {
 		return NULL;
 	}
 
-	if (pos >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
 	UWSGI_RELEASE_GIL
+        int ret = uwsgi_sharedarea_read8(id, pos, &byte);
+        UWSGI_GET_GIL
 
-	uwsgi_wlock(uwsgi.sa_lock);
-
-	char value = uwsgi.sharedarea[pos];
-
-	uwsgi_rwunlock(uwsgi.sa_lock);
-
-	UWSGI_GET_GIL
-
-	return PyInt_FromLong(value);
+        if (ret) {
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_read8()");
+        }
 
+	return PyInt_FromLong(byte);
 }
 
 PyObject *py_uwsgi_sharedarea_read(PyObject * self, PyObject * args) {
+	int id;
 	uint64_t pos = 0;
-	uint64_t len = 1;
+	uint64_t len = 0;
 
-	if (uwsgi.sharedareasize <= 0) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-
-	if (!PyArg_ParseTuple(args, "l|l:sharedarea_read", &pos, &len)) {
+	if (!PyArg_ParseTuple(args, "il|l:sharedarea_read", &id, &pos, &len)) {
 		return NULL;
 	}
 
-	if (pos + len >= uwsgi.page_size * uwsgi.sharedareasize) {
-		Py_INCREF(Py_None);
-		return Py_None;
+	if (!len) {
+		struct uwsgi_sharedarea *sa = uwsgi_sharedarea_get_by_id(id, pos);
+                if (!sa) {
+			return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_read()");	
+                }
+                len = (sa->max_pos+1)-pos;
 	}
 
 	PyObject *ret = PyString_FromStringAndSize(NULL, len);
@@ -1652,18 +1679,51 @@ PyObject *py_uwsgi_sharedarea_read(PyObject * self, PyObject * args) {
 #endif
 
 	UWSGI_RELEASE_GIL
+        int64_t rlen = uwsgi_sharedarea_read(id, pos, storage, len);
+        UWSGI_GET_GIL
 
-	uwsgi_wlock(uwsgi.sa_lock);
+        if (rlen < 0) {
+		Py_DECREF(ret);
+                return PyErr_Format(PyExc_ValueError, "error calling uwsgi_sharedarea_read()");
+        }
 
-	memcpy(storage, uwsgi.sharedarea + pos, len);
-
-	uwsgi_rwunlock(uwsgi.sa_lock);
-
-	UWSGI_GET_GIL
+	// HACK: we are safe as rlen can only be lower or equal to len
+	Py_SIZE(ret) = rlen;
 
 	return ret;
 }
 
+#if defined(PYTHREE) || defined(Py_TPFLAGS_HAVE_NEWBUFFER)
+#ifndef HAS_NOT_PyMemoryView_FromBuffer
+PyObject *py_uwsgi_sharedarea_memoryview(PyObject * self, PyObject * args) {
+        int id;
+	if (!PyArg_ParseTuple(args, "i:sharedarea_memoryview", &id)) {
+                return NULL;
+        }
+	struct uwsgi_sharedarea *sa = uwsgi_sharedarea_get_by_id(id, 0);
+	if (!sa) {
+        	return PyErr_Format(PyExc_ValueError, "cannot get a memoryview object from sharedarea %d", id);
+        }
+	Py_buffer info;
+	if (PyBuffer_FillInfo(&info, NULL, sa->area, sa->max_pos+1, 0, PyBUF_CONTIG) < 0)
+        	return PyErr_Format(PyExc_ValueError, "cannot get a memoryview object from sharedarea %d", id);
+	return PyMemoryView_FromBuffer(&info);
+}
+#endif
+
+PyObject *py_uwsgi_sharedarea_object(PyObject * self, PyObject * args) {
+	int id;
+        if (!PyArg_ParseTuple(args, "i:sharedarea_object", &id)) {
+                return NULL;
+        }
+        struct uwsgi_sharedarea *sa = uwsgi_sharedarea_get_by_id(id, 0);
+        if (!sa) {
+                return PyErr_Format(PyExc_ValueError, "cannot get an object from sharedarea %d", id);
+        }
+	return (PyObject *) sa->obj;
+}
+#endif
+
 PyObject *py_uwsgi_spooler_freq(PyObject * self, PyObject * args) {
 
 	if (!PyArg_ParseTuple(args, "i", &uwsgi.shared->spooler_frequency)) {
@@ -2519,11 +2579,11 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 	{"ready", py_uwsgi_ready, METH_VARARGS, ""},
 
 	{"set_user_harakiri", py_uwsgi_set_user_harakiri, METH_VARARGS, ""},
-	//{"call_hook", py_uwsgi_call_hook, METH_VARARGS, ""},
 
 	{"websocket_recv", py_uwsgi_websocket_recv, METH_VARARGS, ""},
 	{"websocket_recv_nb", py_uwsgi_websocket_recv_nb, METH_VARARGS, ""},
 	{"websocket_send", py_uwsgi_websocket_send, METH_VARARGS, ""},
+	{"websocket_send_binary", py_uwsgi_websocket_send_binary, METH_VARARGS, ""},
 	{"websocket_handshake", py_uwsgi_websocket_handshake, METH_VARARGS, ""},
 
 	{"chunked_read", py_uwsgi_chunked_read, METH_VARARGS, ""},
@@ -2538,11 +2598,25 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 static PyMethodDef uwsgi_sa_methods[] = {
 	{"sharedarea_read", py_uwsgi_sharedarea_read, METH_VARARGS, ""},
 	{"sharedarea_write", py_uwsgi_sharedarea_write, METH_VARARGS, ""},
-	{"sharedarea_readbyte", py_uwsgi_sharedarea_readbyte, METH_VARARGS, ""},
-	{"sharedarea_writebyte", py_uwsgi_sharedarea_writebyte, METH_VARARGS, ""},
-	{"sharedarea_readlong", py_uwsgi_sharedarea_readlong, METH_VARARGS, ""},
-	{"sharedarea_writelong", py_uwsgi_sharedarea_writelong, METH_VARARGS, ""},
-	{"sharedarea_inclong", py_uwsgi_sharedarea_inclong, METH_VARARGS, ""},
+	{"sharedarea_readbyte", py_uwsgi_sharedarea_read8, METH_VARARGS, ""},
+	{"sharedarea_writebyte", py_uwsgi_sharedarea_write8, METH_VARARGS, ""},
+	{"sharedarea_read8", py_uwsgi_sharedarea_read8, METH_VARARGS, ""},
+	{"sharedarea_write8", py_uwsgi_sharedarea_write8, METH_VARARGS, ""},
+	{"sharedarea_readlong", py_uwsgi_sharedarea_read64, METH_VARARGS, ""},
+	{"sharedarea_writelong", py_uwsgi_sharedarea_write64, METH_VARARGS, ""},
+	{"sharedarea_read64", py_uwsgi_sharedarea_read64, METH_VARARGS, ""},
+	{"sharedarea_write64", py_uwsgi_sharedarea_write64, METH_VARARGS, ""},
+	{"sharedarea_inclong", py_uwsgi_sharedarea_inc64, METH_VARARGS, ""},
+	{"sharedarea_inc64", py_uwsgi_sharedarea_inc64, METH_VARARGS, ""},
+	{"sharedarea_rlock", py_uwsgi_sharedarea_rlock, METH_VARARGS, ""},
+	{"sharedarea_wlock", py_uwsgi_sharedarea_wlock, METH_VARARGS, ""},
+	{"sharedarea_unlock", py_uwsgi_sharedarea_unlock, METH_VARARGS, ""},
+#if defined(PYTHREE) || defined(Py_TPFLAGS_HAVE_NEWBUFFER)
+#ifndef HAS_NOT_PyMemoryView_FromBuffer
+	{"sharedarea_memoryview", py_uwsgi_sharedarea_memoryview, METH_VARARGS, ""},
+#endif
+	{"sharedarea_object", py_uwsgi_sharedarea_object, METH_VARARGS, ""},
+#endif
 	{NULL, NULL},
 };
 
